// maindlg.h : interface of the CMainDlg class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_MAINDLG_H__2B437259_B355_4109_A274_653E4A2E66F3__INCLUDED_)
#define AFX_MAINDLG_H__2B437259_B355_4109_A274_653E4A2E66F3__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

class CMainDlg : public CDialogImpl<CMainDlg>, CUpdateUI<CMainDlg>,
		public CMessageFilter, public CIdleHandler
{
public:
	enum { IDD = IDD_MAINDLG };

	virtual BOOL PreTranslateMessage(MSG* pMsg)
	{
		return IsDialogMessage(pMsg);
	}

	virtual BOOL OnIdle()
	{
		return FALSE;
	}

	BEGIN_MSG_MAP(CMainDlg)
		MESSAGE_HANDLER(WM_INITDIALOG, OnInitDialog)
		COMMAND_ID_HANDLER(ID_APP_ABOUT, OnAppAbout)
		COMMAND_ID_HANDLER(IDOK, OnOK)
		COMMAND_ID_HANDLER(IDCANCEL, OnCancel)
		COMMAND_ID_HANDLER(IDC_REFRESH, OnRefresh)
		MESSAGE_HANDLER(WM_SYSCOMMAND, OnSysCommand)
	END_MSG_MAP()

	BEGIN_UPDATE_UI_MAP(CMainDlg)
	END_UPDATE_UI_MAP()

	LRESULT OnInitDialog(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		// center the dialog on the screen
		CenterWindow();

		// set icons
		HICON hIcon = (HICON)::LoadImage(_Module.GetResourceInstance(), MAKEINTRESOURCE(IDR_MAINFRAME), 
			IMAGE_ICON, ::GetSystemMetrics(SM_CXICON), ::GetSystemMetrics(SM_CYICON), LR_DEFAULTCOLOR);
		SetIcon(hIcon, TRUE);
		HICON hIconSmall = (HICON)::LoadImage(_Module.GetResourceInstance(), MAKEINTRESOURCE(IDR_MAINFRAME), 
			IMAGE_ICON, ::GetSystemMetrics(SM_CXSMICON), ::GetSystemMetrics(SM_CYSMICON), LR_DEFAULTCOLOR);
		SetIcon(hIconSmall, FALSE);

		// Add "About..." menu item to system menu.

		// IDM_ABOUTBOX must be in the system command range.
		_ASSERTE((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
		_ASSERTE(IDM_ABOUTBOX < 0xF000);

		CMenu SysMenu = GetSystemMenu(FALSE);
		if(::IsMenu(SysMenu))
		{
			TCHAR szAboutMenu[256];
			if(::LoadString(_Module.GetResourceInstance(), IDS_ABOUTBOX, szAboutMenu, 255) > 0)
			{
				SysMenu.AppendMenu(MF_SEPARATOR);
				SysMenu.AppendMenu(MF_STRING, IDM_ABOUTBOX, szAboutMenu);
			}
		}
		SysMenu.Detach();

		// register object for message filtering and idle updates
		CMessageLoop* pLoop = _Module.GetMessageLoop();
		pLoop->AddMessageFilter(this);
		pLoop->AddIdleHandler(this);

		UIAddChildWindowContainer(m_hWnd);

		// Subclass and initialize the list control
		m_CROTList.Attach(GetDlgItem(IDC_ROTLIST));
		DWORD dwStyleEx = m_CROTList.GetExtendedListViewStyle();
		m_CROTList.SetExtendedListViewStyle(dwStyleEx | LVS_EX_GRIDLINES);
		CRect rcList;
		m_CROTList.GetClientRect(&rcList);
		m_CROTList.InsertColumn(0,"Type",LVCFMT_LEFT,rcList.Width()/4,-1);
		m_CROTList.InsertColumn(1,"Display Name",LVCFMT_LEFT,3*(rcList.Width()/4),-1);
		if ( FAILED(Refresh()) ) {
			// Could not update the list...
			m_CROTList.DeleteAllItems();
			MessageBox(_T("Error loading ROT values..."),_T("Running Object Table Viewer"),MB_OK|MB_ICONERROR);
		} // if

		return TRUE;
	}

	LRESULT OnAppAbout(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		CAboutDlg dlg;
		dlg.DoModal();
		return 0;
	}

	LRESULT OnOK(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		// TODO: Add validation code 
		CloseDialog(wID);
		return 0;
	}

	LRESULT OnCancel(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		CloseDialog(wID);
		return 0;
	}

	void CloseDialog(int nVal)
	{
		DestroyWindow();
		::PostQuitMessage(nVal);
	}

	LRESULT OnSysCommand(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& bHandled)
	{
		UINT uCmdType = (UINT)wParam;

		if((uCmdType & 0xFFF0) == IDM_ABOUTBOX)
		{
			CAboutDlg dlg;
			dlg.DoModal();
		}
		else
			bHandled = FALSE;

		return 0;
	}

	LRESULT OnRefresh(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		if ( FAILED(Refresh()) ) {
			// Could not update the list...
			m_CROTList.DeleteAllItems();
			MessageBox(_T("Error loading ROT values..."),_T("Running Object Table Viewer"),MB_OK|MB_ICONERROR);
		} // if
		return 0;
	}

	// Helpers
protected:
	HRESULT Refresh()
	{
		// Clear the list
		m_CROTList.DeleteAllItems();
		m_CROTList.SetRedraw(FALSE);

		CComPtr<IRunningObjectTable> spROT;
		HRESULT hr = S_OK;
		LPOLESTR postrDisplayName = NULL;
		try {
			hr = GetRunningObjectTable(0,&spROT);
			if ( FAILED(hr) ) throw hr;
			if ( spROT.p == NULL ) throw E_POINTER;

			CComPtr<IEnumMoniker> spEnumMoniker;
			hr = spROT->EnumRunning(&spEnumMoniker);
			if ( FAILED(hr) ) throw hr;
			if ( spEnumMoniker.p == NULL ) throw E_POINTER;

			// Allocate memory for the display name
			postrDisplayName = (LPOLESTR)CoTaskMemAlloc(1024);
			if ( postrDisplayName == NULL ) throw E_POINTER;

			// Enumerate the ROT items
			CComPtr<IMoniker> spMoniker;
			CComPtr<IBindCtx> spBindCtx;
			CString strType;
			USES_CONVERSION;
			while ( TRUE ) {
				// Obtain the next object moniker
				spMoniker = NULL;
				hr = spEnumMoniker->Next(1,&spMoniker,NULL);
				if ( hr == S_FALSE ) break; // S_FALSE means done enumerating...
				if ( FAILED(hr) ) throw hr;
				if ( spMoniker.p == NULL ) throw E_POINTER;

				// Pull moniker type information
				DWORD dwMksys = MKSYS_NONE;
				hr = spMoniker->IsSystemMoniker(&dwMksys);
				if ( FAILED(hr) ) throw hr;
				long iItemNum = 0;
				switch ( dwMksys ) {
					case MKSYS_NONE:
						strType = "Custom";
						break;

					case MKSYS_GENERICCOMPOSITE:
						strType = "Composite";
						break;

					case MKSYS_FILEMONIKER:
						strType = "File";
						break;

					case MKSYS_ANTIMONIKER:
						strType = "Anti";
						break;

					case MKSYS_ITEMMONIKER:
						strType = "Item";
						break;

					case MKSYS_POINTERMONIKER:
						strType = "Pointer";
						break;

					case MKSYS_CLASSMONIKER:
						strType = "Class";
						break;

					default:
						strType = "Unknown";
						break;

				} // switch

				// Insert the moniker type into the list
				m_CROTList.InsertItem(iItemNum,(LPCTSTR)strType);

				// Create a binding context (req'd for call to
				// IMoniker::GetDisplayName()).
				spBindCtx = NULL;
				hr = CreateBindCtx(0,&spBindCtx);
				if ( FAILED(hr) ) throw hr;

				// Now obtain the display name
				hr = spMoniker->GetDisplayName(spBindCtx,NULL,&postrDisplayName);
				if ( FAILED(hr) ) throw hr;

				// Insert display name into the list, second
				// column...
				m_CROTList.AddItem(iItemNum++,1,OLE2T(postrDisplayName));
			} // while

		} // try
		catch(HRESULT hrErr) {
			// Some COM error
			hr = hrErr;
			ATLTRACE("ROTView: COM error, %#08x\n",hrErr);
		} // catch
		catch(...) {
			// Some error
			hr = E_UNEXPECTED;
			ATLTRACE("ROTView: application error, %#08x\n",hr);
		} // catch

		// Free the display string
		CoTaskMemFree(postrDisplayName);

		// Enable display
		m_CROTList.SetRedraw(TRUE);

		return hr;
	}

	// Controls
	CContainedWindowT<WTL::CListViewCtrl> m_CROTList;
};


/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_MAINDLG_H__2B437259_B355_4109_A274_653E4A2E66F3__INCLUDED_)
